/*
**************** This version Modified 11/3/2017 by George Thomas, W5JDX for use with TFT LCD heads up display ***********
* Unnecessary code for this particular usage was removed, and some additional code was added.
* Thanks to James Buck, VE3BUX for the original FT857D Arduino library
* ************************************************************************************************************************
  FT857D.cpp	Arduino library for controlling a Yaesu FT857D
			radio via CAT commands.
 
 Version:  0.1
 Created:  2012.08.16
Released:  2012.08.17
  Author:  James Buck, VE3BUX
     Web:  http://www.ve3bux.com

*/

#include <Arduino.h>
#include <SoftwareSerial.h>
#include "FT857D.h"

// define software serial IO pins here:
// Note I am using an Arduino Mega.
// some pins won't work with SoftwareSerial on the Mega.
extern SoftwareSerial rigCat(52,53); // arduino rx(rig TX),ard tx(rig RX)

#define dlyTime 5	// delay (in ms) after serial writes

FT857D::FT857D(){ }	// nothing to do when first instanced

// Setup software serial with user defined input
// from the Arduino sketch (function, though very slow)
void FT857D::setSerial(SoftwareSerial portInfo) {
  rigCat = portInfo;
}

// similar to Serial.begin(baud); command
void FT857D::begin(int baud) {
  rigCat.begin(baud);
}

byte FT857D::getMode() {
	unsigned long l = getFreqMode();
	return mode;
}

// get the frequency and the current mode
// if called as getFreqMode() return only the frequency
unsigned long FT857D::getFreqMode() {
	byte rigGetFreq[5] = {0x00,0x00,0x00,0x00,0x00};
	rigGetFreq[4] = CAT_RX_FREQ_CMD; // command byte
	byte chars[4];
	long timeout = millis();
	long elapsed = 0;

	rigCat.flush();	// clear the RX buffer which helps prevent
				// any crap data from making it through

	sendCmd(rigGetFreq, 5);
	
	while (rigCat.available() < 5 && elapsed < 100) {
		elapsed = millis() - timeout;
     	;}
	
	for (int j = 0; j < 4; j++) {
		chars[j] = rigCat.read();  
   	}

	mode = rigCat.read(); 
	freq = from_bcd_be(chars, 8);
	return freq;
}

//-------------added by George, W5JDX---------------
// read RX state and return S-meter val (Squelch, CTCSS/DCS, Disc. Ctr, S-Meter)
int FT857D::getSmeter() {
  byte rigRXState[5] = {0x00,0x00,0x00,0x00,0x00};
  rigRXState[4] = CAT_RX_DATA_CMD;
  
  rigCat.flush(); // clear the RX buffer which helps prevent
        // any crap data from making it through

  sendCmd(rigRXState, 5);
  sLevel = getByte();
 
  if (sLevel == 255){  //255 when transmitting so return 0.
    sLevel = 0;}

  return sLevel;
}
//--------------------------------------------------

// determine if the radio is in TX state
// unless the radio is actively TX, the result is always
// 0x255 so any value other than 0x255 means TX !
boolean FT857D::chkTX() {
	byte rigTXState[5] = {0x00,0x00,0x00,0x00,0x00};
	rigTXState[4] = CAT_TX_DATA_CMD;
  rigCat.flush(); // clear the RX buffer which helps prevent
        // any crap data from making it through
	
	sendCmd(rigTXState, 5);
	byte reply = getByte();

	if (reply == 255) {  // not transmitting
		return false;
	}
	else {               // transmitting
		return true;
	}
}

// return TX Power **************** Needs Work ************************
// unless the radio is actively TX, the result is always
// 0x255 so any value other than 0x255 means TX !
int FT857D::chkTXpwr() {
  byte rigTXState[5] = {0x00,0x00,0x00,0x00,0x00};
  rigTXState[4] = CAT_TX_DATA_CMD;
  rigCat.flush(); // clear the RX buffer which helps prevent
        // any crap data from making it through
  
  sendCmd(rigTXState, 5);
  int watts = getByte();
  if (watts == 255)
    watts = 0;
  else
    watts = watts * 10;
  
  return watts;
}

// spit out any DEBUG data via this function
void FT857D::comError(char * string) {
 Serial.println("Communication Error!");
 Serial.println(string);
}

// gets a byte of input data from the radio
byte FT857D::getByte() {
	unsigned long startTime = millis();
  //while (rigCat.available() < 1 && millis() < startTime + 2000) {
	while (rigCat.available() < 1 && millis() < startTime + 100) {
		;
	}
	byte radioReply = rigCat.read();
	return radioReply ;
}

// this is the function which actually does the 
// serial transaction to the radio
void FT857D::sendCmd(byte cmd[], byte len) {
	for (byte i=0; i<len; i++) {
		rigCat.write(cmd[i]);
	}
//	return getByte();	// should make this work more quickly
					// in a future update
}

// this function reduces total code-space by allowing for
// single byte commands to be issued (ie. all the toggles)
byte FT857D::singleCmd(int cmd) {
	byte outByte[5] = {0x00,0x00,0x00,0x00,0x00};
	outByte[4] = cmd;
	sendCmd(outByte, 5);
	return getByte();
}

// send a single byte of data (will be removed later)
void FT857D::sendByte(byte cmd) {
	rigCat.write(cmd);
}

void FT857D::flushRX() {
	rigCat.flush();
}

// GPL
// taken from hamlib work
unsigned long FT857D::from_bcd_be(const  byte bcd_data[], unsigned bcd_len)
{
	int i;
	long f = 0;

	for (i=0; i < bcd_len/2; i++) {
		f *= 10;
		f += bcd_data[i]>>4;
		f *= 10;
		f += bcd_data[i] & 0x0f;
	}
	if (bcd_len&1) {
		f *= 10;
		f += bcd_data[bcd_len/2]>>4;
	}
	return f;
}


